// This file is part of the brownmove simulation package
// 
// (C) 2003 - 2009 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove


#ifndef EXTERNALSHAPE_H
#define EXTERNALSHAPE_H

#include <iostream>
#include <string>
#include <sstream>
#include <vector>

#include "shape.h"
#include "types.h"
#include "constants.h"
#include "vectormath.h"


namespace brown{
	/**
	 *@author tg
	 */

	//! This shape provides a hook to apply an external force onto the particles.

	//! @todo Include externalshpe in the parser, supply some simple external fields (const, harmonic potential)

class ExternalShape : public Shape {

	public:
	ExternalShape();
	ExternalShape(const ExternalShape &other);
	~ExternalShape();

	/** returns some string for testing */
	virtual std::string describe();
	
	// virtual bool checkCollision(const position &p);
	/** restores the parts' (absolute and relative) positions from fullPosition */
	virtual void recalculate(const fullPosition &p);
	
	//! adds another hook to an external force with position and function pointer. 
	//! Also calculates the relative position of the hook from the given molecule center.
	void addHook(externalPos ep, const position &moleculeCenter);
	
	
// 	/** set the pointer to an analytic function describing the interaction between a wall and a molecule */
// 	void setInteraction(functionPointer &fp) {
// 		interAction = fp;
// 	}

// 	//! return the function pointer to an analytical interaction
// 	functionPointer getInteraction() {
// 		return(interAction);
// 	}

	
	
	//! calculate the forces and torques from my hooks and the respective function
	void calculateImpact(fullForce &myForce);
	
	//! returns the number of hooks
	int size();
	
	  
	private:
	//! Positions of the hooks in the particle coordinate system
	vector<externalPos> hooks;
	//! Absolute positions of the hooks in the simulation volume
	vector<externalPos> actual_hooks;
};
}

#endif
