// This file is part of the brownmove simulation package
// 
// (C) 2003 - 2009 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove


#ifndef RANDOM_H
#define RANDOM_H


#include <sys/types.h>
#include <time.h>
#include <stdio.h>
#include <math.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

// Choose one of the following RNG algorithms:
// 
// maximally equidistributed combined Tausworthe generator by L'Ecuyer
#define RNG_TYPE gsl_rng_taus2
// 
// MT19937 generator of Makoto Matsumoto and Takuji Nishimura
// #define RNG_TYPE gsl_rng_mt19937



namespace brown {
	//! @author tg
	
	//! Wrappers for the random number stuff from the Gnu Scientific Library. 
	//! The global RNG is defined here.
	
	//! @name Initialisation
	//! @{
	// 
	//! Allocate and init the global RNG from system time
	void init_ran();
	//! Seed the RNG without allocating a new one
	void random_seed(unsigned long int s);
	//! Clean up RNG
	void exit_ran();
	//! @}
	
	
	//! @name The different random distributions
	//! @{
	// 
	//! Linear distribution from [0,1)
	double ran2();
	
	//! Exponential distribution acc. exp(-t/t0) for dissociation reactions.
	//! t0 = 1/rate constant.
	double exp_ran(double &t0);
	
	//! Normal distributed random numbers acc. exp(-(t-t0)^2 / (2*sigma^2)).
	double gaussran();
	//! @}
	
	
} /* namespace brown */

#endif /* RANDOM_H */
