// This file is part of the brownmove simulation package
// 
// (C) 2003 - 2009 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove


#ifndef VDWSHAPE_H
#define VDWSHAPE_H

#include <vector>
#include <iostream>
#include <string>
#include <sstream>

#include "types.h"
#include "constants.h"
#include "vectormath.h"

#include "shape.h"

// #include "wall.h"


namespace brown{
	
	/**
	 *@author tg
	 */
	
	//! This class handled short-ranged van-der-Waal type interactions
	class vdWShape : public Shape {
		
		public:
		//! Constructor
		vdWShape();
		//! Copy Constructor
		vdWShape(const vdWShape &other);
		//! Destructor
		~vdWShape();
		
		
		//! returns some string for testing
		virtual std::string describe();

		//! restores the parts' (absolute and relative) positions from fullPosition
		virtual void recalculate(const fullPosition &p);

		//! Adds another sphere with position and radius to the vector, 
		//! also calculates the relative position of the sphere from the molecules center given
		void addSphere(vdWPos vp, const position &moleculeCenter);
		
		//! adds another vdWBlock
		void addBlock(vdWBlock vb);
		
		
		//! calculate the forces and torques from the charges of this and the other vdWShape
		void calculateImpact(fullForce &myForce, fullForce &otherForce, vdWShape *vs, const bool isPeriodic[3], const doublevec &cellSize);

		//! set the pointer to the array oof vdW-parameter-structs
		void setParameterArray(vdWParameter **theArray) {
			paramsArray = theArray;
		}
							   		
		//! set a vdw-wall: offset and color
		void setWall(const double &offset, const long int &clr) {
			myWallColor = clr;
			myWallOffset = offset;
		}
		
		//! What's the color of my wall (-1 for none)
		const long int getWallColor() {
			return(myWallColor);
		}
		
		//! Get (pointer to the) actual position of the vdw wall
		position *getWallPosition() {
			return(&myActualWallPos);
		}
		
		//! Get (pointer to the) actual normal of the vdw wall
		doublevec *getWallNormal() {
			return(&myActualWallNormal);
		}
		
		//! returns the number of spheres
		int size();
		
		//! Adds it contribution to the center of mass of the gestalt
		void addMyCM(position &theCM, double &theMass);
		
		//! Dump info about vdW spheres in browndef format
		void dumpStructure();
		

		// this is my private stuff!!! Mooff!
		private:
		vector<vdWPos> spheres;
		vector<vdWPos> actual_spheres;
		vector<vdWBlock> blocks;
		vector<vdWBlock> actual_blocks;
		long int myWallColor;			// there may be only one vdW wall per vdWShape
		double myWallOffset;			// displacement of the vdw wall relative to the gestalt's position
		position myActualWallPos;		// position of this one wall
		doublevec myActualWallNormal;	// normal vector pointing off the wall
		vdWParameter **paramsArray; 
	};
}

#endif
