// This file is part of the brownmove simulation package
// 
// (C) 2003 - 2009 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Project Homepage http://service.bioinformatik.uni-saarland.de/brownmove


#include "externalshape.h"

namespace brown{
	
	ExternalShape::ExternalShape() {
	}
	
	ExternalShape::ExternalShape(const ExternalShape &other){
		hooks = other.hooks;
		actual_hooks = other.actual_hooks;
	}
	
	ExternalShape::~ExternalShape(){
		hooks.clear();
		actual_hooks.clear();
	}
	
	
	std::string ExternalShape::describe(){
		ostringstream ausgabe("", ios::out);
		int i=0;
		
		ausgabe << "externalshape actual positions";
		for(vector<externalPos>::iterator mlc = actual_hooks.begin(); mlc != actual_hooks.end(); ++mlc, i++){
			ausgabe << " : "<<(*mlc).pos[0]<<" "<<(*mlc).pos[1]<<" "<<(*mlc).pos[2]<<"  "<<(*mlc).dr[0]<<" "<<(*mlc).dr[1]<<" "<<(*mlc).dr[2] ; 
		}
		return(ausgabe.str());
	}
	

	void ExternalShape::recalculate(const fullPosition &p) {
		// cout << "**** externalshape:recalculate" << endl;
		
		vector<externalPos>::iterator act = actual_hooks.begin();
		for(vector<externalPos>::iterator mlc = hooks.begin(); mlc != hooks.end(); ++mlc, ++act)
		{
			doublevec_set((*act).pos, (*mlc).pos);
			doublevec_rotate((*act).pos, p.rot);
			positionMove((*act).pos, p.pos);
			doublevec_set((*act).dr, (*act).pos);
			doublevec_sub((*act).dr, p.pos);
		}
		
	}

	void ExternalShape::addHook(externalPos cp, const position &moleculeCenter){
		if(cp.interaction) {
			doublevec_set(cp.dr, cp.pos);
			doublevec_sub(cp.dr, moleculeCenter);
			hooks.push_back(cp);
			actual_hooks.push_back(cp);
		} else {
			cerr << "ExternalShape::addHook: interaction is not set (=NULL)" << endl;
			exit(-1);
		}
	}
	
	
	void ExternalShape::calculateImpact(fullForce &myForce)
	{
		force df;
		torque dt;
		
		// loop over all my hooks
		for(vector<externalPos>::iterator mlc = actual_hooks.begin(); mlc != actual_hooks.end(); ++mlc)
		{
			// cout <<"*** ExternalShape::calculateImpact  mlc: "<< "   " << (*mlc).pos[0] << "  " << (*mlc).pos[1] << "  " << (*mlc).pos[2] << endl;
			
			// get force vector on hook according to external function
			(*mlc).interaction(df, (*mlc).pos);
							   
			// add force to previous result
			doublevec_add(myForce.f, df);
			
			// calculate and add torque
			doublevec_kreuzprod(dt, (*mlc).dr, df);
			doublevec_add(myForce.t, dt);
		}
	}
	
			
	
	int ExternalShape::size() {
		return(hooks.size());
	}
	
	
}
